import numpy as np
import MLCourse.utilities as utils


################################################################
############Q1: Regression###################
#########################################
class Regressor:
    """
    Generic regression interface; returns random regressor
    Random regressor randomly selects w from a Gaussian distribution
    """
    def __init__(self, parameters = {}):
        self.params = parameters
        self.weights = None

    def getparams(self):
        return self.params

    def learn(self, Xtrain, ytrain):
        # Learns using the traindata
        self.weights = np.random.rand(Xtrain.shape[1])

    def predict(self, Xtest):
        # Most regressors return a dot product for the prediction
        ytest = np.dot(Xtest, self.weights)
        return ytest

    def l2err(self, prediction, y):
        """ l2 error (i.e., root-mean-squared-error) RMSE http://statweb.stanford.edu/~susan/courses/s60/split/node60.html"""
        return np.linalg.norm(np.subtract(prediction, y)) / np.sqrt(y.shape[0])


class MeanPredictor(Regressor):
    """
    Returns the average target value observed; a reasonable baseline
    """
    def __init__(self, parameters = {}):
        self.params = parameters
        self.mean = None

    def learn(self, Xtrain, ytrain):
        # Learns using the traindata
        self.mean = np.mean(ytrain)

    def predict(self, Xtest):
        return np.ones((Xtest.shape[0],))*self.mean

    def get_learned_params(self):
        return {"Mean":self.mean}


# Graduate admissions dataset: ~1.0, Adagrad: ~0.8
class LinearRegression(Regressor):
    def __init__(self, parameters):
        self.params = parameters
        self.weights = None
        self.transfer = utils.sigmoid
        self.dtransfer = utils.dsigmoid
        self.stepsize_approach = self.params['stepsize_approach']
        self.epochs = self.params['num_epochs']
        self.batch_size = self.params['batch_size']

    def learn(self, X, y):
        pass

    def getStepSize(self, gradinet):
        """
        returns step size based on the approach chosen
        """
        pass

    def predict(self, Xtest):
        """
        Most regressors return a dot product for the prediction
        """
        pass


# Graduate admissions dataset: ~1.0, AdaGrad: ~0.7
class PolynomialRegression(LinearRegression):
    """docstring for Po"""
    def __init__(self, parameters):
        pass

    def learn(self, X, y):
        pass

    def predict(self, Xtest):
        pass


################################################################
############Q3: Classification###################
#########################################

# Susy: ~50 error
class Classifier:
    def __init__(self, parameters = {}):
        """ Params can contain any useful parameters for the algorithm """
        self.params = {}

    def getparams(self):
        return self.params

    def learn(self, Xtrain, ytrain):
        """ Learns using the training data """
        pass

    def predict(self, Xtest, threshold=0.5):
        probs = np.random.rand(Xtest.shape[0])
        ytest = utils.threshold_probs(probs, threshold=threshold)
        return ytest


# Susy: ~24 error
class LogisticRegression(Classifier):
    def __init__(self, parameters):
        self.params = parameters
        self.weights = None
        self.transfer = utils.sigmoid
        self.dtransfer = utils.dsigmoid
        self.epochs = self.params['epochs']
        self.batch_size = self.params['batch_size']

    def learn(self, X, y):
        """
        implements SGD updates
        """
        pass

    def predict(self, Xtest, threshold=0.5):
        pass


# Susy: ~22 error
class PolynomialLogisticRegression(LogisticRegression):
    """docstring for Po"""
    def __init__(self, parameters):
        super(PolynomialLogisticRegression, self).__init__(parameters)

    def learn(self, X, y):
        pass

    def predict(self, Xtest):
        pass
